@@ -1,3 +1,18 @@
+# SYS imports
+import os
+import sys
+
+# Append subdirectories to sys.path
+ppath = os.path.abspath(os.path.join(os.path.dirname(__file__),os.pardir))
+print(ppath)
+qpath = os.path.dirname(__file__)
+print(qpath)
+fpath = os.path.join(os.path.dirname(__file__), 'devices')
+sys.path.append(qpath)
+sys.path.append(fpath)
+print(sys.path)
+
+# Old imports for API Calls
 import requests
 from requests.exceptions import HTTPError
 
@@ -9,18 +24,18 @@ import yaml
 from pathlib import Path
 
 # New imports to optimize API-Calls
+from typing import Dict
 import asyncio
 import aiohttp
 import urllib.error
 import urllib.parse
 import urllib.request
-from typing import Any, Dict, List
-
 
 # Import from lupulib
-import lupulib
+# import lupulib
 import lupulib.devices
 import lupulib.constants as CONST
+import lupulib.exceptions
 # from lupulib.exceptions import LupusecParseError, LupusecRequestError, LupusecResponseError
 # from lupulib.devices.binary_sensor import LupusecBinarySensor
 # from lupulib.devices.switch import LupusecSwitch
@@ -28,6 +43,7 @@ import lupulib.constants as CONST
 
 _LOGGER = logging.getLogger(__name__)
 home = str(Path.home())
+print(home)
 
 
 class LupusecAPI:
@@ -35,62 +51,88 @@ class LupusecAPI:
 
     def __init__(self, username, password, ip_address) -> None:
         """LupusecAPI constructor to interface Lupusec Alarm System."""
-        
         self._username = username
         self._password = password
         self._ip_address = ip_address
         self._url = "http://{}/action/".format(ip_address)
         self._model = "unknown"
         self._auth = None
-        if _username != None and _password != None:
-            self._auth = aiohttp.BasicAuth(login=_username, password=_password, encoding='utf-8')
-        self._session = aiohttp.ClientSession(auth=_auth)
+        if self._username != None and self._password != None:
+            self._auth = aiohttp.BasicAuth(login=self._username, password=self._password, encoding='utf-8')
+        self._session = aiohttp.ClientSession(auth=self._auth)
         self._system = None
 
 
-    async def _async_api_call(client, url):
+    async def _async_api_call(client, url) -> Dict:
         """Generic sync method to call the Lupusec API"""
         _LOGGER.debug("_async_api_call() called: ")
 
         try:
             async with client.get(url) as resp:
-                    # assert resp.status == 200
-                    print(resp.status)
-                    return await resp.text()
-        except aiohttp.ClientResponseError as exception:
-            raise LupusecResponseError(exception.status, exception.message) from exception
-        except aiohttp.ClientConnectionError as exception:
-            raise LupusecRequestError(str(exception)) from exception
+                _LOGGER.debug("  API-Call: %s", url)
+                if response.status != 200:
+                    _LOGGER.error(f"Response status: {response.status}")
+                    return {}                
+                print(resp.status)
+                content = await resp.json(content_type=None)
+                print("type of response: ", type(content))
+                print("API-Call finished:")
+                return content
+        except aiohttp.client_exceptions.ClientConnectorError:
+            _LOGGER.error("Cannot connect to: ", url)
+            return {}
+
+        except aiohttp.ContentTypeError:
+            _LOGGER.error("JSON decode failed")
+            return {}
 
 
-    async def async_get_system(self) -> devices.system.LupusecSystem:
+
+    async def async_get_system(self) -> None:
         """Async method to get the system info."""
         _LOGGER.debug("async_get_system() called: ")
 
         # Login to Lupusec System
-        url_cmd = CONST.LOGIN_REQUEST
-        async with _session as client:
-            data = await _async_api_call(client, url_cmd)
-        _LOGGER.debug(data)
+        #url_cmd = CONST.LOGIN_REQUEST
+        #_LOGGER.debug("  API-Call: %s", url_cmd)
+        #async with self._session as client:
+        #    data = await _async_api_call(client, url_cmd)
+        #    _LOGGER.debug(data)
 
         # Get System Info
-        url_cmd = CONST.INFO_REQUEST
-        async with _session as client:
-            data = await _async_api_call(client, url_cmd) 
-    
-        # try to parse json response
-        try:
-            json_data = json.loads(data)["updates"]
-        except json.JSONDecodeError as exception:
-            raise LupusecParseError(str(exception)) from exception
-        
-        _LOGGER.debug(json_data)
-        print("  Hardware-Version: %s ", json_data["rf_ver"])
-        print("  Firmware-Version: %s ", json_data["em_ver"])
+        #url_cmd = CONST.INFO_REQUEST
+        #_LOGGER.debug("  API-Call: %s", url_cmd)
+        #async with self._session as client:
+        #    data = await _async_api_call(client, url_cmd) 
+        #    _LOGGER.debug(data)
+        #    return devices.system.LupusecSystem(data)
+
+        # new approach
+        async with self._session as client:
+            tasks = []
+
+            # LOGIN_REQUEST
+            tasks.append(asyncio.ensure_future(_async_api_call(client, CONST.LOGIN_REQUEST)))
+            # LOGIN_REQUEST
+            tasks.append(asyncio.ensure_future(_async_api_call(client, CONST.INFO_REQUEST)))
+
+            response_list = await asyncio.gather(*tasks)
+            for content in response_list:
+                print(content)
+                _LOGGER.debug("Response Content:", content)
+                # return devices.system.LupusecSystem(content)
+
+
+            # try to parse json response
+            # try:
+            #     json_data = json.loads(data)["updates"]
+            #    _LOGGER.debug(json_data)
+            #    print("  Hardware-Version: %s ", json_data["rf_ver"])
+            #    print("  Firmware-Version: %s ", json_data["em_ver"])
+            #    return await devices.system.LupusecSystem(json_data)
+            #except json.JSONDecodeError as exception:
+            #    raise LupusecParseError(str(exception)) from exception
 
-        #return self.clean_json(response.text)[CONST.INFO_HEADER]
-        return devices.system.LupusecSystem(json_data)
- 
 
     def _request_post(self, action, params={}):
         return self.session.post(
